#!/bin/csh -f
#PBS -e en_cons.err
#PBS -o en_cons.log
#PBS -l nodes=1:ppn=3
#PBS -l walltime=1:00:00
#PBS -m ae
# Job name:
#PBS -N test_en_cons_pot
#PBS -r n


# if running as PBS job, change into the correct directory (the directory containing this file)
if ($?PBS_O_WORKDIR) then
cd $PBS_O_WORKDIR
endif

set pb = 16
set tp = 8
set num_blocks = 100

printf "Physics test: energy conservation. VaryPotential\n"

set dt = 0.0064
set w_interval = 400
set block_size =  40000



(../../../../UserPrograms/md_pair  --pot_type=1 --numIOBlocks=${num_blocks} --IOBlockSize=${block_size} --pb=$pb --tp=$tp --dt=${dt} --thermostat=0 --device=0 --dump_nrg=${w_interval} --writeTraj=0 --write_v_com_x=1 --mom_reset=0 --out_prec=9 > en_cons${dt}.log) >& en_cons${dt}.err
zcat TrajectoryFiles/energies*.dat.gz > energies${dt}.dat

rm -rf TrajectoryFiles


grep "Processing time GPU" en_cons*.log > cpu_times.log
echo "To plot the energies, do: xmgrace -block energies0.0064.dat -bxy 0:6 

echo "To calculate energy fluctuations and drift, do: python ../Analysis.py [results will be in stdE_vs_dt.dat]"


