#ifndef INTEGRATOR_H
#define INTEGRATOR_H

/*
  Copyright (C) 2010  Thomas Schrøder
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  LICENSE.txt file for license details.
*/

class ParticleData; class KernelPlan; class IntegratorMetaData;

#include "rumd/Sample.h"

// The abstract Integrator base class. Inherit this class to build your own integrator.
class Integrator{
 
 private:
  Integrator(const Integrator&); 
  Integrator& operator=(const Integrator&); 
  
 protected:
  Sample* S; ParticleData* P; KernelPlan kp;  
  float timeStep;
  
  // Override this function to allocate numParticles dependent memory.
  virtual void AllocateIntegratorState(){}; 
  
 public:
  Integrator() : S(0), P(0), kp(), timeStep(1.f){}
  virtual ~Integrator(){};
  
  virtual void SetSampleData(Sample* S, ParticleData* data){
    this->S = S;
    kp = S->GetKernelPlan();
    kp.threads.y = 1; 
    P = data; 
    AllocateIntegratorState();
  }

  virtual void SetIntegratorState(IntegratorMetaData* itg_metaData) { throw RUMD_Error("Integrator", "SetIntegratorState", "Has not been implemented for this Integrator type yet");}
  
  virtual void Integrate() = 0;

  // Override this function to call functions AFTER the force calculation, 
  // but invisible to the user program.
  virtual void CalculateAfterForce(){};

  virtual void SetTimeStep(float dt){ timeStep = dt; };
  virtual float GetTimeStep() const { return timeStep; };
  virtual double GetKineticEnergy() const { return -1.f; };
};

#endif // INTEGRATOR_H
