#include "rumd/ConfigurationMetaData.h"

#include <cstring>

void ConfigurationMetaData::ReadMetaData(char *commentLine, bool verbose) {

  int nItems;
  unsigned int index;
  // for tokenizing: there is a two-level hierarchy of tokens,
  // (in some cases, eg list-arguments, three-level)
  // so we need to use strtok_r to allow nested tokenizing
  char *token;
  char *keyword;
  char *argument;
  char *listItem; // for list-arguments (comma-separated items)
  char *tokenMarker1; // highest level tokens
  char *tokenMarker2; // for separating keyword from argument
  char *tokenMarker3; // for list-arguments
	
  // possible entries in the columns list: 
  char requiredCols[][5] = {"type","x","y","z"};
  unsigned int numReq = sizeof(requiredCols)/(5*sizeof(char));
  char imStr[][5] = {"imx","imy","imz"};
  char velStr[][3] = {"vx","vy","vz"};
  char forStr[][3] = {"fx","fy","fz"};

  // top-level
  token = strtok_r(commentLine, " \t", &tokenMarker1);
  while(token) {
    // now token marks the next major token
    keyword = strtok_r(token, "=", &tokenMarker2);
    argument =  strtok_r(0, "=", &tokenMarker2);
    if(strcmp(keyword, "ioformat") == 0) {
      nItems = sscanf(argument, "%d", &rumd_conf_ioformat);
      if (verbose) printf("read ioformat %d\n",rumd_conf_ioformat);
    }
    else if (strcmp(keyword, "timeStepIndex") == 0) {
      nItems = sscanf(argument, "%d", &(timeStepIndex));
      if (verbose) printf("read timeStepIndex %d\n",timeStepIndex);
    }
    else if (strcmp(keyword, "logLin") == 0) {
      listItem = strtok_r(argument, ",", &tokenMarker3); 
      nItems = sscanf(listItem, "%lu",&(logLin.block));
      if (verbose) printf("read    logLin.block %lu\n", logLin.block);
      listItem = strtok_r(0, ",", &tokenMarker3); 
      nItems = sscanf(listItem, "%lu",&(logLin.base));
      if (verbose) printf("read    logLin.base %lu\n", logLin.base);
      listItem = strtok_r(0, ",", &tokenMarker3);
      nItems = sscanf(listItem, "%lu",&(logLin.index));
      if (verbose) printf("read    logLin.index %lu\n", logLin.index);
      listItem = strtok_r(0, ",", &tokenMarker3);
      nItems = sscanf(listItem, "%lu",&(logLin.maxIndex));
      if (verbose) printf("read    logLin.maxIndex %lu\n", logLin.maxIndex);
      listItem = strtok_r(0, ",", &tokenMarker3); 
      nItems = sscanf(listItem, "%lu",&(logLin.maxInterval));
      if (verbose) printf("read    logLin.maxInterval %lu\n", logLin.maxInterval);
      assert(0 == strtok_r(0, ",", &tokenMarker3));
      if (verbose)
	printf("read conf_index: Block %lu, Index %lu, MaxIndex %lu\n",
	       logLin.block, logLin.index, logLin.maxIndex);
      has_logLin = true;
    }
    else if (strcmp(keyword, "numTypes") == 0) {
      nItems = sscanf(argument, "%d", &numTypes);
      assert(numTypes <= MaxNumTypes);
      if (verbose) printf("read numTypes %d\n", numTypes);
    }
    else if (strcmp(keyword, "Nose-Hoover-Ps") == 0) {
      nItems = sscanf(argument, "%f", &(Ps));
      if (verbose) printf("Read Nose-Hoover-Ps %f\n", Ps);
    }
    else if (strcmp(keyword, "Barostat-Pv") == 0) {
      nItems = sscanf(argument, "%f", &(Pv));
      if (verbose) printf("read Barostat-Pv %f\n", Pv);
    }
    else if (strcmp(keyword, "dt") == 0) {
      nItems = sscanf(argument, "%f", &dt);
      if (verbose) printf("read time step (dt) %f\n", dt);
    }		
    else if (strcmp(keyword, "mass") == 0) {
      listItem = strtok_r(argument, ",", &tokenMarker3);
      index = 0;
      while(listItem) {
	assert(index < numTypes);
	nItems = sscanf(listItem, "%f",&(massOfType[index]));
	index++;
	listItem = strtok_r(0, ",", &tokenMarker3);
      }
      
      if (verbose) printf("masses");
      for(index = 0; index < numTypes; ++index)
	if (verbose) printf(" %f", massOfType[index]);
      if (verbose) printf("\n");
    }
    else if (strcmp(keyword, "symbol") == 0) {
      listItem = strtok_r(argument, ",", &tokenMarker3);
      index = 0;
      while(listItem) {
	assert(index < numTypes);
	// not doing anything with symbol yet
	// (should include it in meta-data, but have to worry about
	// allocating space for potentially arbitrary length strings,
	// or make sure to read a maximum number of characters)
	if (verbose) printf("symbol %d is %s\n",index,listItem);
	index++;
	listItem = strtok_r(0, ",", &tokenMarker3);
      }
    }
    else if (strcmp(keyword, "boxLengths") == 0) {
      listItem = strtok_r(argument, ",", &tokenMarker3);
      unsigned int cd = 0;
      while(listItem) {
	assert(cd < DIM);
	nItems = sscanf(listItem, "%f",&(L[cd]));
	cd++;
	listItem = strtok_r(0, ",", &tokenMarker3);
      }
      if (verbose) printf("boxLengths");
      for(unsigned int cd = 0; cd < DIM; ++cd)
	if (verbose) printf(" %f", L[cd]);
      if (verbose) printf("\n");
    }
    else if (strcmp(keyword, "numMolecules") == 0) {
      
    }
    else if (strcmp(keyword, "columns") == 0) {
      listItem = strtok_r(argument, ",", &tokenMarker3);
      index = 0;
		
      
      while(listItem && index < numReq) {
	assert(strcmp(listItem, requiredCols[index]) == 0);
	 if (verbose) printf("columns, have required column %d (%s)\n",index,listItem);
	index++;
	listItem = strtok_r(0, ",", &tokenMarker3);
      }
      while(listItem) {
	// check if index < numReq (not all required columns present)?
	if(strcmp(listItem, "imx") == 0){
	  for(index = 0; index < DIM; ++index) {
	    assert(strcmp(listItem, imStr[index]) == 0);
	    if (verbose) printf("columns: have images: %s\n", listItem);
	    listItem = strtok_r(0, ",", &tokenMarker3);
	  }
	  has_images = true;
	}			
	else if(strcmp(listItem, "vx") == 0) {
	  for(index = 0; index < DIM; index++) {
	    assert(strcmp(listItem, velStr[index]) == 0);
	    if (verbose) printf("columns: have velocities %s\n", listItem);
	    listItem = strtok_r(0, ",", &tokenMarker3);
	  }
	  has_velocities = true;
	}
	else if(strcmp(listItem, "fx") == 0) {
	  for(index = 0; (index < DIM) ; index++) {
	    assert(strcmp(listItem, forStr[index]) == 0);
	    if (verbose) printf("columns: have forces %s\n", listItem);
	    listItem = strtok_r(0, ",", &tokenMarker3);
	  }
	  has_forces = true;
				}
	else if(strcmp(listItem, "pe") == 0) {
	  if (verbose) printf("columns: have pe\n");
	  has_pot_energies = true;
	  listItem = strtok_r(0, ",", &tokenMarker3);
	}
	else if(strcmp(listItem, "vir") == 0) {
	  has_virials = true;
	  if (verbose) printf("columns: have vir\n");
	  listItem = strtok_r(0, ",", &tokenMarker3);
				}
      } // end while (listItem) [columns]
    }
    else {
      fprintf(stderr, "ConfMetaReader: unknown keyword %s\n", keyword);
      exit(1);
    }
    
    // get the next major token
    token = strtok_r(0, " \t", &tokenMarker1);
  }
}
